<?php
namespace GartentechnikCom;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class EinstellungsRegister {
	static private $einstellungen = [];
	static private $gtcom_einstellungs_cache = [];
	
	# VOID
	public static function init() {
		self::$einstellungen = get_option('gartentechnik_com_einstellungen');
		self::$gtcom_einstellungs_cache = get_option('gartentechnik_com_gtcom_einstellungs_cache');
	}
	# SCALAR
	public static function gib_einstellung($name) {
		$val = @self::$einstellungen[$name];
		if(is_null($val)) {
			$val = '';
		}
		return $val;
	}
	# BOOLEAN
	public static function ist_gtcom_slug_aktiv($key) {
		foreach(self::gib_gtcom_pfade() as $e) {
			if($e['key'] == $key) {
				if($e['seiten_id'] > 0 && get_post_status($e['seiten_id']) == 'publish') {
					return true;
				}
				
				return ($e['ist_aktiv'] ? true : false);
			}
		}
		return true;
	}
	# STRING || NULL
	public static function gib_gtcom_key_fuer_pfad($path) {
		foreach(self::gib_gtcom_pfade() as $e) {
			if(preg_match('/^\/' . preg_quote($e['slug'], '/') . '(?:\/|$)/', $path) === 1) {
				if($e['seiten_id'] > 0 && get_post_status($e['seiten_id']) == 'publish') {
					return null;
				}
				return $e['key'];
			}
		}
		return null;
	}
	# STRING || NULL
	public static function gib_gtcom_slug_fuer_key($key) {
		foreach(self::gib_gtcom_pfade() as $e) {
			if($e['key'] == $key) {
				return $e['slug'];
			}
		}
		return null;
	}
	# ARRAY
	public static function gib_gtcom_pfade() {
		if(!is_array(self::$gtcom_einstellungs_cache['pfade'])) {
			return [];
		}
		return self::$gtcom_einstellungs_cache['pfade'];
	}
	# VOID
	public static function finish($response) {
		$gtcom_einstellungen = $response->einstellungen();
		if(!$gtcom_einstellungen || !$gtcom_einstellungen['ist_ok']) {
			return;
		}
		if($gtcom_einstellungen['checksum'] === self::$gtcom_einstellungs_cache['checksum']) {
			return;
		}
		
		$cache = self::$gtcom_einstellungs_cache;
		$cache['checksum'] = $gtcom_einstellungen['checksum'];
		$cache['pfade'] = $gtcom_einstellungen['pfade'];
		$cache['weiterleitungen'] = $gtcom_einstellungen['weiterleitungen'];
		update_option('gartentechnik_com_gtcom_einstellungs_cache', $cache, true);
	}
	# STRING || NULL
	public static function gib_pfad_weiterleitung($pfad) {
		if(!is_array(self::$gtcom_einstellungs_cache['weiterleitungen'])) {
			return null;
		}
		foreach(self::$gtcom_einstellungs_cache['weiterleitungen'] as $e) {
			if(preg_match('/^\/' . preg_quote($e['quell_pfad'], '/') . '(?:\/|$)/', $pfad) === 1) {
				return '/' . $e['ziel_pfad'];
			}
		}
		return null;
	}
}
