<?php
namespace GartentechnikCom;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

class MotoristPlugin {
	public static $response = null;
	private $domain = null;
	private $url_schema = null;
	private $current_path_prefix = null;
	private $is_currently_retrying_slug_generation = false;

	# VOID
	function __construct() {
	}
	# VOID
	public function init($url_schema, $domain, $current_path_prefix) {
		$this->domain = $domain;
		$this->url_schema = $url_schema;
		$this->current_path_prefix = $current_path_prefix;
	}
	# BOOLEAN
	public function ist_gtcom_request($request_method, $path) {
		return $this->_ist_gtcom_pfad($path);
	}
	# BOOLEAN
	private function _ist_gtcom_pfad($path) {
		$key = \GartentechnikCom\EinstellungsRegister::gib_gtcom_key_fuer_pfad($path);
		if(is_null($key)) {
			return false;
		}
		
		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv($key)) {
			return true;
		}
		
		return false;
	}
	# STRING
	private function _uebersetze_pfad($path) {
		$key = \GartentechnikCom\EinstellungsRegister::gib_gtcom_key_fuer_pfad($path);
		$path = preg_replace('/^\/[^\/]+/', "/$key", $path);
		return $path;
	}
	# OBJ
	public function execute_gtcom_request(
		$request_method, $path, $cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
	) {
		$path = $this->_uebersetze_pfad($path);
		$api_response = $this->_call_api(
			true, $request_method, '/api' . $path, $cgi_parameter,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		self::$response = new Response();
		if(is_null($api_response)) {
			self::$response->is_404(true);
			return self::$response;
		}
		
		$this->_fill_response(self::$response, $api_response);
		return self::$response;
	}
	# OBJ
	public function execute_other(
		$cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
	) {
		if(!$ist_admin_kontext) $rolle_des_aktuellen_users = '';

		$api_response = $this->_call_api(
			true, 'GET', '/api/', $cgi_parameter,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		self::$response = new Response();
		if(is_null($api_response)) {
			self::$response->is_404(true);
			return self::$response;
		}
		
		$this->_fill_response(self::$response, $api_response);
		return self::$response;
	}
	# ARRAY
	public function gib_suchergebnisse($suche) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		$list = $this->_call_api(
			false, 'GET', '/json/suche', array(s => $suche),
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($list)) return array();
		
		return $list;
	}
	# ARRAY
	public function sende_anfrage($param) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		
		$path = '/json/anfrage';
		if(@$param['ist_adminanfrage']) {
			$path = '/json/adminbereich_anfrage';
		}
		$result = $this->_call_api(
			false, 'POST', $path, $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($result)) {
			return array(
				'error' => '1',
				"antwort" => array(
					"versendet" => '0',
				),
			);
		}
		return $result;
	}
	# ARRAY
	public function sende_produktrezension($param) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		$result = $this->_call_api(
			false, 'POST', '/json/produkt-rezension/', $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		if(is_null($result)) {
			return array(
				'error' => '1',
				"antwort" => array(
					"versendet" => '0',
				),
			);
		}
		return $result;
	}
	# ARRAY || NULL
	public function gib_widget_produktansicht($ids) {
		$ist_admin_kontext = 0;
		$ist_superadmin_kontext = 0;
		$rolle_des_aktuellen_users = 'user';
		gartentechnik_com_debug('produkt_widget_start');
		$result = $this->_call_api(
			false, 'POST', "/json/produktwidget", $ids,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('produkt_widget_end');
		if(is_null($result)) return array();
		
		return $result;
	}
	# STRING
	private function _lese_datei($file) {
		$fh = fopen($file, "r");
		if(!$fh) return '';
		
		$content = fread($fh, filesize($file));
		fclose($fh);
		
		return $content;
	}
	# ARRAY
	public function sende_admin_api_anfrage(
		$param, $files, $ist_admin_kontext,
		$ist_superadmin_kontext, $rolle_des_aktuellen_users
	) {
		$method = $param['api_method'];
		$path = $param['api_path'];
		unset($param['api_method']);
		unset($param['api_path']);
		if($method != 'GET' && $method != 'POST' && $method != 'PUT' && $method != 'DELETE') {
			$method = 'GET';
		}
		$path = preg_replace('/[^a-z\/_\-]/', '', $path);
		$sub_path = $ist_superadmin_kontext ? 'superadmin' : 'motorist_admin';
		gartentechnik_com_debug('admin_api_start');
		
		if($files && array_key_exists('file', $files) && is_array($files['file'])) {
			gartentechnik_com_debug('upload_file');
			$param['file.name'] = $files['file']['name'];
			$param['file.mime_type'] = $files['file']['type'];
			if(!$files['file']['error']) {
				$content = $this->_lese_datei($files['file']['tmp_name']);
			}
			$param['file.content_base64'] = base64_encode($content);
		}
		$result = $this->_call_api(
			false, $method, "/json/$sub_path/$path", $param,
			$ist_admin_kontext, $ist_superadmin_kontext, $rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('admin_api_end');
		if(is_null($result)) {
			return array( 'error' => '1' );
		}
		return $result;
	}
	# STRING
	private function _gib_client_ip() {
		if(getenv('HTTP_X_FORWARDED_FOR')) {
			return getenv('HTTP_X_FORWARDED_FOR');
		} elseif(getenv('X_REAL_IP')) {
			return getenv('X_REAL_IP');
		} elseif(getenv('HTTP_CLIENT_IP')) {
			return getenv('HTTP_CLIENT_IP');
		} elseif(getenv('HTTP_X_FORWARDED')) {
			return getenv('HTTP_X_FORWARDED');
		} elseif(getenv('HTTP_FORWARDED_FOR')) {
			return getenv('HTTP_FORWARDED_FOR');
		} elseif(getenv('HTTP_FORWARDED')) {
			return getenv('HTTP_FORWARDED');
		}
		return $_SERVER['REMOTE_ADDR'];
	}
	# HASH
	private function _korrigiere_parameter($cgi_parameter) {
		# Dots and spaces in variable names are converted to underscores. For example <input name="a.b" /> becomes $_REQUEST["a_b"].
		# http://it2.php.net/manual/it/language.variables.external.php
		foreach($cgi_parameter as $key => $value) {
			if(preg_match('/#/', $key)) {
				$neuer_key = str_replace('#', '.', $key);
				$cgi_parameter[$neuer_key] = $value;
				unset($cgi_parameter[$key]);
			}
		}
		return $cgi_parameter;
	}
	# HASH || NULL
	private function _call_api(
		$check_for_404, $request_method, $path, $cgi_parameter,
		$ist_admin_kontext, $ist_superadmin_kontext, $benutzerrolle
	) {
		global $current_user;
		
		if(!$path) $path = '/';

		if(!$benutzerrolle) $benutzerrolle = 'user';

		$api_key = \GartentechnikCom\EinstellungsRegister::gib_einstellung('gartentechnik_com_api_key');
		$api_key = preg_replace('/[^A-F0-9\-]+/', '', $api_key);
		$cgi_parameter['wordpress_api_key'] = $api_key;
		if(!$api_key) {
			return null;# Kein API-Key, kein Keks!
		}

		if(
			($ist_admin_kontext || $ist_superadmin_kontext)
			&& @$cgi_parameter['page']
			&& preg_match(
				'/^gartentechnik_com\-admin\-?(.*)$/',
				sanitize_text_field(@$cgi_parameter['page']),
				$page_treffer
			)
		) {
			$sub_path = $page_treffer[1];
			if(!$sub_path) $sub_path = '';

			$path = '/api/motorist_admin/' . $sub_path;
			if($ist_superadmin_kontext) {
				$path = '/api/superadmin/' . $sub_path;
			}
			unset($cgi_parameter['page']);
		}

		$api_url = 'https://www.gartentechnik.com';
		$compress = true;
		$ist_externes_plugin = true;
		if(getenv('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT')) {
			$compress = false;
			$api_url = 'http://app:8080';
			$ist_externes_plugin = false;
		}
		
		$response_ist_standardseite = false;
		if($request_method == 'GET' && $path == '/api/') {
			$response_ist_standardseite = true;
		}
		
		$api_timeout = 60;
		$timeout_ist_verkuerzt = false;
		if($response_ist_standardseite && $this->_anzahl_der_api_timeouts_der_letzten_5_minuten() >= 2) {
			gartentechnik_com_debug('api-timeout wird verkuerzt');
			error_log('api-timeout fuer Standardseiten wird verkuerzt');
			$api_timeout = 2;
			$timeout_ist_verkuerzt = true;
		}
		
		# https://developer.wordpress.org/reference/classes/wp_http/request/
		$api_version = \GartentechnikCom\Register::gib_api_version();
		gartentechnik_com_debug('gtcom_post_start');
		$wp_benutzerloginname = '';
		if($ist_admin_kontext && $current_user && $current_user->exists()) {
			$wp_benutzerloginname = $current_user->user_login . '(' . $current_user->user_email . ')';
		}
		$request = array(
			'method'				=> 'POST',
			'timeout'				=> $api_timeout,
			'httpversion'			=> '1.1',
			'compress'				=> $compress,
			'decompress'			=> true,
			'headers'				=> array(
				'X-Ist-Externes-Plugin'			=> ($ist_externes_plugin ? '1' : '0'),
				'X-Wordpress-Client-IP'			=> $this->_gib_client_ip(),
				"X-Wordpress-Host"				=> $this->domain,
				"X-Wordpress-Schema"			=> $this->url_schema,
				"X-Wordpress-PathPrefix"		=> $this->current_path_prefix,
				"X-Wordpress-Request-Method"	=> $request_method,
				"X-Wordpress-User-Agent"		=> getenv('HTTP_USER_AGENT'),
				"X-Wordpress-Ist-Admin-Kontext"	=> ($ist_admin_kontext ? '1' : '0'),
				"X-Wordpress-Benutzerrolle"		=> $benutzerrolle,
				"X-Wordpress-Plugin-Version"	=> \GartentechnikCom\Register::gib_plugin_version(),
				"X-Wordpress-Benutzerlogin"		=> $wp_benutzerloginname,
				"Accept-Encoding"				=> "gzip",
				"Content-Type"					=> "application/x-www-form-urlencoded",
				"User-Agent"					=> "gt2-wp-plugin-v" . \GartentechnikCom\Register::gib_plugin_version(),
			),
			'sslverify'				=> true,
			'redirection'			=> 0,
			'blocking'				=> true,
			'reject_unsafe_urls'	=> false,
			'limit_response_size'	=> 0,
			'body'					=> $this->_korrigiere_parameter($cgi_parameter),
		);
		$response = wp_remote_post($api_url . '/api/motorist_plugin/v' . $api_version . $path, $request);
		$response_ist_fehlerhaft = is_wp_error($response);
		$api_response = [
			'body'				=> $response_ist_fehlerhaft ? '' : $response['body'],
			'code'				=> $response_ist_fehlerhaft ? 0 : $response['response']['code'],
			'ist_fehlerhaft'	=> $response_ist_fehlerhaft,
		];
		unset($response);# RAM wieder freigeben... das Ding ist riesig
		gartentechnik_com_debug('gtcom_post_end');
		
		
		if($api_response['ist_fehlerhaft']) {
			gartentechnik_com_debug(
				"API-Fehler, Req:" . print_r($request, true)
			);
			error_log("GTCOM-API-Fehler, Req:" . print_r($request, true));
			
			if(!$timeout_ist_verkuerzt) {
				$this->_speichere_api_timeoutfehler();
			}
			
			if($response_ist_standardseite) {
				$api_response = &$this->_lese_cache_fuer_standardseite();# Referenz... um Speicher zu sparen
				if(!$api_response) {
					return null;
				}
				gartentechnik_com_debug("Benutze Cache der Standardseite");
				error_log('Benutze Cache der Standardseite');
			} else {
				return null;
			}
		} else {
			if($response_ist_standardseite) {
				gartentechnik_com_debug('Schreibe Cache der Standardseite');
				$this->_schreibe_cache_fuer_standardseite($api_response);
			}
		}
		
		gartentechnik_com_debug('gtcom_post_json_start');
		$result = json_decode($api_response['body'], true);
		gartentechnik_com_debug('gtcom_post_json_end');
		if($check_for_404) {
			$result['ist_404'] = ($api_response['code'] == 404 ? true : false);
		}
		return $result;
	}
	# INT
	private function _anzahl_der_api_timeouts_der_letzten_5_minuten() {
		$fehler = get_option('gartentechnik_com_api_timeoutfehler');
		if(!is_array($fehler)) return 0;
		
		$now = time();
		$count = 0;
		foreach($fehler as $zeitpunkt_des_fehlers) {
			if($zeitpunkt_des_fehlers + (5 * 60) > $now) {
				$count++;
			}
		}
		return $count;
	}
	# VOID
	private function _speichere_api_timeoutfehler() {
		$alte_fehler = get_option('gartentechnik_com_api_timeoutfehler');
		if(!is_array($alte_fehler)) $alte_fehler = [];
		
		$now = time();
		$neue_fehler = [$now];
		foreach($alte_fehler as $zeitpunkt_des_fehlers) {
			if($zeitpunkt_des_fehlers + (10 * 60) >= $now) {# nur 6 Minuten speichern
				array_push($neue_fehler, $zeitpunkt_des_fehlers);
			}
		}
		update_option('gartentechnik_com_api_timeoutfehler', $neue_fehler, true);
	}
	# VOID
	private function _schreibe_cache_fuer_standardseite(&$response) {# Referenz, um Speicher zu sparen
		update_option('gartentechnik_com_standardseite_cache', $response, false);# Kein Autoload, weil nur im Notfall noetig
	}
	# OBJ || NULL
	private function _lese_cache_fuer_standardseite() {
		return get_option('gartentechnik_com_standardseite_cache');
	}
	# OBJ
	private function _fill_response($response, $content) {
		$response->template_key(@$content['global']['template_key']);
		
		$response->is_404(@$content['ist_404']);
		$response->content_type(@$content['http_header']['content_type']);
		$response->charset(@$content['http_header']['charset']);
		$response->redirect_location(@$content['http_header']['weiterleitung']);

		$response->html_head_titel(@$content['seite']['title']);
		$response->html_head_content(@$content['seite']['head']);
		$response->html_head_allgemein_content(@$content['seite']['allgemeiner_head']);
		$response->html_head_metadescription(@$content['seite']['metadescription']);
		$response->html_content_titel(@$content['seite']['h1']);
		$response->main_content(@$content['seite']['body'], @$content['seite']['presenter_key']);
        $response->html_footer_content(@$content['seite']['footer']);
		$response->content_is_plain(@$content['seite']['is_plain']);

		$response->widget_oeffnungszeiten(@$content['widget']['oeffnungszeiten']);
		$response->widget_kontaktformular(@$content['widget']['kontaktformular']);
		$response->widget_newsletter_abonnieren(@$content['widget']['newsletter-abonnieren']);
		$response->widget_haendleradresse(@$content['widget']['haendleradresse']);
		$response->widget_angebote(@$content['widget']['angebote']);
		$response->widget_angesagte_kategorien(@$content['widget']['angesagte-kategorien']);
		$response->widget_servicegebiet(@$content['widget']['servicegebiet']);
		$response->widget_lieferanten(@$content['widget']['lieferanten']);
		$response->allgemeine_html_header(@$content['widget']['allgemeine_html_header']);
		$response->kategoriebaum(json_decode(@$content['widget']['kategoriebaum'], true));
		$response->admin_menue(json_decode(@$content['widget']['admin_menue'], true));
		$response->einstellungen(json_decode(@$content['widget']['einstellungen'], true));

		return $response;
	}
	# STRING
	public function generate_unique_post_slug($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug) {
		if($this->is_currently_retrying_slug_generation) {
			return $slug;
		}

		if($this->_ist_gtcom_pfad("/$slug/")) {
			$hit = array();
			$number = 2;
			if(preg_match('/\-(\d+)$/', $slug, $hit) === 1) {
				$number = $hit[1] + 1;
				$slug = preg_replace('/-\d+$/', '', $slug);
			}
			while($this->_ist_gtcom_pfad("/$slug-$number/")) {
				$number++;
				if($number > 10000) break;
			}

			$this->is_currently_retrying_slug_generation = true;
			while(true) {
				if($number > 15000) break;

				$new_gtcom_slug = "$slug-$number";
				$new_wp_slug = wp_unique_post_slug($new_gtcom_slug, $post_ID, $post_status, $post_type, $post_parent);
				if($new_wp_slug == "$new_gtcom_slug-2") {
					$number++;
					continue;
				}

				break;
			}
			$slug = $new_wp_slug;
			$this->is_currently_retrying_slug_generation = false;
		}
		return $slug;
	}
}
?>
