<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/*
Version:     2.0.6
Plugin Name: Gartentechnik.com
Plugin URI:  https://wordpress.gartentechnik.com/
Description: Gartentechnik.com ist ein kostenfreies, open source WordPress-Plugin zur Benutzung der kostenpflichtigen API des gleichnamigen Service für Motorgerätefachhändler und Motoristen.
Text Domain: gartentechnik-com
Author:      Gartentechnik.communications GmbH
Author URI:  https://www.gartentechnik.com/
License:     GPL3
License URI: https://www.gnu.org/licenses/gpl-3.0.html
*/

$gartentechnik_com_domainname = 'gartentechnik-com';
$gartentechnik_debug_request_id = rand();
$gartentechnik_debug_start_zeit = microtime();
$gartentechnik_com_pluginpfad = __FILE__;
$gartentechnik_com_api_version = '2';

# VOID
function gartentechnik_com_debug($msg) {
	global $gartentechnik_debug_request_id;
	global $gartentechnik_debug_start_zeit;
	trigger_error(
		"GTCOM[" . $gartentechnik_debug_request_id . '/' . (microtime() - $gartentechnik_debug_start_zeit) . '] ' . $msg,
		E_USER_NOTICE
	);
}

include_once(__DIR__ . '/_rss_import.php');

add_filter('init', function() use($gartentechnik_com_domainname, $gartentechnik_com_api_version) {
	gartentechnik_com_debug('init_start! req:' . @$_SERVER['REQUEST_URI'] . ' referrer:' . @$_SERVER['HTTP_REFERER']);
	
	if(getenv('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT') && preg_match('/^\/fotos\/sites\//', getenv('REQUEST_URI'))) {
		gartentechnik_com_debug("abort; internal image content");
		return;
	}
	
	$url_path_starts_with = function($haystack, $needle) {
		return (substr($haystack, 0, strlen($needle)) === $needle);
	};
	if(
		$url_path_starts_with(getenv('REQUEST_URI'), home_url('/wp-content', 'relative'))
		||
		$url_path_starts_with(getenv('REQUEST_URI'), home_url('/wp-includes', 'relative'))
	) {
		gartentechnik_com_debug("abort; internal content");
		return;
	}
	
	$domain = $_SERVER['HTTP_HOST'];
	$url_schema = $_SERVER['HTTPS'] == 'on' ? 'https' : 'http';
	if(preg_match('/^(https?):\/\/([^\/]+)/', home_url('/'), $wp_url_hits)) {
		$domain = $wp_url_hits[2];
		$url_schema = $wp_url_hits[1];
	}
	
	$current_path = $_SERVER['REQUEST_URI'];
	$current_path_prefix = preg_replace('/\/$/', '', home_url('/', 'relative'));
	if(strlen($current_path_prefix) > 0) {
		$current_path = substr($current_path, strlen($current_path_prefix));
	}
	gartentechnik_com_debug("HOST: '$url_schema' + '$domain' + '$current_path_prefix' + '$current_path'");
	
	load_plugin_textdomain($gartentechnik_com_domainname, false, dirname( plugin_basename(__FILE__) ) . '/languages/' );
	gartentechnik_com_debug('Load language: ' . __('Language: English', $gartentechnik_com_domainname));
	
	require_once(ABSPATH . 'wp-admin/includes/plugin.php');
	require_once(__DIR__ . '/classes/Response.php');
	require_once(__DIR__ . '/classes/MotoristPlugin.php');
	require_once(__DIR__ . '/classes/Register.php');
	require_once(__DIR__ . '/classes/EinstellungsRegister.php');
	require_once(__DIR__ . '/classes/Linkerzeuger.php');
	require_once(__DIR__ . '/classes/WP/Menue.php');
	
	\GartentechnikCom\EinstellungsRegister::init();

	if(wp_doing_ajax()) {
		$widget_id = @$_GET['widget-id'] ? sanitize_text_field(@$_GET['widget-id']) : sanitize_text_field(@$_POST['widget-id']);
		if(!$widget_id) $widget_id = '';
		$action = @$_GET['action'] ? sanitize_text_field(@$_GET['action']) : sanitize_text_field(@$_POST['action']);
		if(!$action) $action = '';
		if(
			array_key_exists('s', $_POST)
			&& count(sanitize_text_field($_POST['s'])) > 0
			&& $_SERVER['REQUEST_METHOD'] == 'POST'
			&& preg_match('/ajax_search$/', $action)
		) {
			\GartentechnikCom\Register::ist_such_autovervollstaendigung(true);
		} else if(
			preg_match('/^gartentechnik_com/', $action)
			|| preg_match('/^gartentechnik\-com/', $widget_id)
		) {
			# Weiter normal behandeln
		} else {
			return;
		}
	}

	$ist_superadmin_kontext = (preg_match('/^wp\.gartentechnik\.com(?:\.vm|)$/', $domain) ? true : false);
	if(!$ist_superadmin_kontext) {
		$domain = preg_replace('/^wp(?:|\-test)\./', '', $domain);
	}
	if(preg_match('/^preview\.(.*)\.gartentechnik\.com(.*)$/', $domain, $treffer)) {
		$domain = $treffer[1] . ($treffer[2] ? $treffer[2] : '');
	}

	$current_path = preg_replace('/\?.*$/', '', $current_path);
	$cgi_parameter = $_SERVER['REQUEST_METHOD'] == 'GET' ? $_GET : $_POST;
	$cgi_parameter = stripslashes_deep($cgi_parameter);
	
	$weiterleitungs_pfad = \GartentechnikCom\EinstellungsRegister::gib_pfad_weiterleitung($current_path);
	if($weiterleitungs_pfad) {
		header("HTTP/1.1 301 MovedPermanently");
		header("Location: $weiterleitungs_pfad");
		gartentechnik_com_debug('weiterleitung');
		exit(0);
	}

	gartentechnik_com_debug('rechtepruefung');
	$ist_admin_kontext = false;
	$rolle_des_aktuellen_users = '';
	if(is_user_logged_in()) {
		if(current_user_can('manage_network')) {
			$rolle_des_aktuellen_users = 'admin';
			if($ist_superadmin_kontext) {
				$rolle_des_aktuellen_users = 'superadmin';
			}
		} elseif(current_user_can('manage_options')) {
			$rolle_des_aktuellen_users = 'admin';
		} elseif(current_user_can('edit_pages')) {
			$rolle_des_aktuellen_users = 'editor';
		}
		if($rolle_des_aktuellen_users) {
			$ist_admin_kontext = true;
		}
	}

	$app = new \GartentechnikCom\MotoristPlugin();
	$app->init($url_schema, $domain, $current_path_prefix);
	\GartentechnikCom\Register::init(
		$domain,
		$current_path,
		$app,
		get_plugin_data( __FILE__)['Version'],
		$gartentechnik_com_api_version
	);

	add_filter('robots_txt', function($content) {
		gartentechnik_com_debug('robots_start');
		# Der Google-Bot folgt den Trackingtexten, die die API-URL enthalten und wie ein Linkpfad aussehen, aber keiner sind :(
		$verbotene_pfade = "\nDisallow: /v" . \GartentechnikCom\Register::gib_api_version() . '/api';
		
		$slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('agb');
		if(!$slug) $slug = 'agb';
		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv('agb')) {
			$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$slug/", 'relative');
		}
		
		$slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('datenschutz');
		if(!$slug) $slug = 'datenschutz';
		if(\GartentechnikCom\EinstellungsRegister::ist_gtcom_slug_aktiv('datenschutz')) {
			$verbotene_pfade = $verbotene_pfade . "\nDisallow: " . home_url("/$slug/", 'relative');
		}
		
		$kaufen_slug = \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen');
		gartentechnik_com_debug('robots_end');
		return $content
			. $verbotene_pfade
			. "\nCrawl-delay: 1"
			. "\nSitemap: " . home_url("/$kaufen_slug/sitemap.xml") . "\n";
	}, 1000, 1);
	add_filter('wp_unique_post_slug', function($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug) use ($app) {
		gartentechnik_com_debug('suche_eindeutigen_slug');
		return $app->generate_unique_post_slug($slug, $post_ID, $post_status, $post_type, $post_parent, $original_slug);
	}, 10, 6);

	if(!is_admin() && array_key_exists('s', $cgi_parameter)) {
		\GartentechnikCom\Register::ist_eine_suche(true);
		\GartentechnikCom\Register::suchquery($cgi_parameter['s']);
	}
	gartentechnik_com_debug('request_pruefen_start');
	
	# GTCOM-5653
	if(getenv('GARTENTECHNIK_COM_IS_DOCKER_CONTEXT') && $current_path == "/sitemap.xml") {
		wp_redirect(
			home_url('/' . \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen') . '/sitemap.xml', 'relative')
		);
		gartentechnik_com_debug('sitemap wird gerufen');
		exit(0);
	} else if($app->ist_gtcom_request($_SERVER['REQUEST_METHOD'], $current_path)) {
		gartentechnik_com_debug('gtcom_request_start');
		#header("X-GTCOM-DEBUG-TYPE: 1/" . \GartentechnikCom\Register::gib_api_version());
		$response = $app->execute_gtcom_request(
			$_SERVER['REQUEST_METHOD'],
			$current_path,
			$cgi_parameter,
			$ist_admin_kontext,
			$ist_superadmin_kontext,
			$rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('gtcom_request_ende');
		\GartentechnikCom\Register::zeige_gtcom(true);
		remove_filter('the_content', 'wpautop');# Sonst fuegt WP um viele Elemente im Content ein P-Tag
		if($response->is_404() || $response->redirect_location()) {
			\GartentechnikCom\Register::zeige_gtcom(false);
			gartentechnik_com_debug('404');
			#header("X-GTCOM-DEBUG-404: 1");
			add_filter('request', function($request) {
				return array('error' => '404');
			}, 20, 1);
		} elseif($response->content_is_plain()) {
			gartentechnik_com_debug('plaintext');
			#header("X-GTCOM-DEBUG-PLAIN: 1");
			header("HTTP/1.1 200 OK");
			header('Content-Type: ' . $response->content_type() . "; charset=" . $response->charset());
			echo $response->main_content();
			exit(0);
		}
	} else {
		gartentechnik_com_debug('wp_request_start');
		#header("X-GTCOM-DEBUG-TYPE: 2/" . \GartentechnikCom\Register::gib_api_version());
		$response = $app->execute_other(
			$cgi_parameter,
			$ist_admin_kontext,
			$ist_superadmin_kontext,
			$rolle_des_aktuellen_users
		);
		gartentechnik_com_debug('wp_request_end');
		if(\GartentechnikCom\Register::ist_eine_suche()) {
			$response->redirect_location(
				home_url('/' . \GartentechnikCom\EinstellungsRegister::gib_gtcom_slug_fuer_key('kaufen') . '/', 'relative')
				. '?s=' . urlencode(\GartentechnikCom\Register::suchquery())
			);
			gartentechnik_com_debug('suche_ausfuehren');
			add_filter('request', function($request) {
				return array('error' => '404');
			}, 20, 1);
		}
	}

	require_once(__DIR__ . '/_allgemeine_funktionen.php');
	require_once(__DIR__ . '/_autovervollstaendigung_suchfeld.php');
	require_once(__DIR__ . '/_produktsuche_fuer_produktwidget.php');
	require_once(__DIR__ . '/_produktwidget_datenabruf.php');
	require_once(__DIR__ . '/_zeige_gt_inhalte_inline.php');
	if(!$ist_superadmin_kontext) {
		require_once(__DIR__ . '/_menu_post_type.php');
	}
	require_once(__DIR__ . '/_gtcom_adminbereich.php');
	
	if(is_user_logged_in()) {
		if(!extension_loaded('xml')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "xml" um abonnierte RSS-Nachrichten importieren und Angebots-Widgets anzeigen zu können. Bitte installieren Sie dieses Modul (bei Debian z.B. php-xml).');
		}
		if(!extension_loaded('pcre')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "pcre" um abonnierte RSS-Nachrichten importieren zu können. Bitte installieren Sie dieses Modul.');
		}
		if(!extension_loaded('iconv')) {
			gartentechnik_com_wp_fehlermeldung('Das Gartentechnik.com-Plugin benötigt das PHP-Modul "iconv" um abonnierte RSS-Nachrichten importieren zu können. Bitte installieren Sie dieses Modul.');
		}
	}

	add_action('widgets_init', function() {
		gartentechnik_com_debug('widget_init_start');
		foreach([
			'Oeffnungszeiten',
			'Kontaktformular',
			'NewsletterAbonnieren',
			'Haendleradresse',
			'Angebote',
			'AngesagteKategorien',
			'Servicegebiet',
			'Lieferanten',
			'Produkt'
		] as $name) {
			require_once(__DIR__ . '/classes/Widget/' . $name . '.php');
			register_widget('\GartentechnikCom\Widget\\' . $name);
		}
		gartentechnik_com_debug('widget_init_end');
	}, 20, 0);

	add_action('wp_head', function() use ($response, $current_path) {
		gartentechnik_com_debug('wp_head_start');
		$zeige_gtcom_content = GartentechnikCom\Register::zeige_gtcom();
		require_once(__DIR__ . '/templates/allgemeine_html_header.html');
		echo $response->html_head_allgemein_content();
		if($zeige_gtcom_content) {
			echo $response->html_head_content();
		}
		gartentechnik_com_debug('wp_head_stop');
	}, 20, 0);
	
	add_action('admin_head', function() use ($response, $current_path) {
		gartentechnik_com_debug('admin_head_start');
		echo $response->html_head_allgemein_content();
		echo $response->html_head_content();
		gartentechnik_com_debug('admin_head_start');
	}, 20, 0);
	
	add_action('wp_footer', function() use ($response) {
		gartentechnik_com_debug('wp_footer_start');
        echo $response->html_footer_content();
		\GartentechnikCom\EinstellungsRegister::finish($response);
		gartentechnik_com_debug('wp_foter_end');
	}, 20, 0);
	add_action('admin_footer', function() use ($response) {
		gartentechnik_com_debug('admin_footer_start');
		echo $response->html_footer_content();
		\GartentechnikCom\EinstellungsRegister::finish($response);
		gartentechnik_com_debug('admin_footer_end');
	}, 20, 0);
	gartentechnik_com_debug('init_end');
}, 0, 0);

?>
